""" call_helper_unittest.py: unit tests for call_helpers

Should be run from python2 and python3!

.. codeauthor:: Christian Herdtweck, christian.herdtweck@intra2net.com
"""

import unittest
import os

import call_helpers
from type_helpers import is_unicode


class CallHelperTester(unittest.TestCase):

    def test_call_and_capture(self):
        """ tests call_and_capture with command ls -a / """

        cmd = ['ls', '-a', '/']
        return_code, out_data, err_data = call_helpers.call_and_capture(cmd)
        self.assertEqual(return_code, 0)
        self.assertEqual(err_data, [])

        # get contents of root dir ourselves
        true_contents = os.listdir('/') + ['.', '..']

        # compare to out_data
        self.assertEqual(sorted(out_data), sorted(true_contents))

        # check type
        self.assertTrue(all(isinstance(entry, str) for entry in out_data),
                        "Output is not of type str!")

    def test_call_and_capture_bytes(self):
        """ tests call_and_capture with command ls -a / """

        cmd = ['ls', '-a', '/']
        return_code, out_data, err_data = \
            call_helpers.call_and_capture(cmd, universal_newlines=False)
        self.assertEqual(return_code, 0)
        self.assertEqual(err_data, [])

        # get contents of root dir ourselves
        true_contents = os.listdir(b'/') + [b'.', b'..']

        # compare to out_data (ignoring order of entries)
        self.assertEqual(sorted(out_data), sorted(true_contents))

        # check type
        self.assertFalse(is_unicode(out_data[0]), "Output is unicode!")

    def test_call_and_capture_err(self):
        """ tests call_and_capture with command ls -a / """

        cmd = ['ls', '-e']
        return_code, out_data, err_data = call_helpers.call_and_capture(cmd)
        self.assertEqual(return_code, 2)
        self.assertEqual(out_data, [])
        self.assertEqual(len(err_data), 2)
        self.assertEqual(err_data[0], "ls: invalid option -- 'e'")
        self.assertEqual(err_data[1], "Try 'ls --help' for more information.")


if __name__ == '__main__':
    unittest.main()
